@@ -67,7 +67,10 @@ typedef struct
 
   GCancellable    *async_ops;
 
+  GoaClient       *goa_client;
+
   /* state flags */
+  gboolean         goa_client_ready;
   gchar          **disabled_sources;
   gint             sources_at_launch;
 
@@ -100,8 +103,10 @@ enum
   SOURCE_ACTIVATED,
   SOURCE_ADDED,
   SOURCE_REMOVED,
+  SOURCE_ENABLED,
   LOAD_COMPLETED,
   QUERY_COMPLETED,
+  GOA_CLIENT_READY,
   NUM_SIGNALS
 };
 
@@ -291,7 +296,8 @@ load_source (GcalManager *manager,
 
   priv = gcal_manager_get_instance_private (manager);
 
-  if (g_hash_table_lookup (priv->clients, source) == NULL)
+  if (g_hash_table_lookup (priv->clients, source) == NULL &&
+      e_source_has_extension (source, E_SOURCE_EXTENSION_CALENDAR))
     {
       /* NULL: because maybe the operation cannot be really cancelled */
       e_cal_client_connect (source,
@@ -666,6 +672,11 @@ gcal_manager_class_init (GcalManagerClass *klass)
                                           NULL, NULL, NULL,
                                           G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+  signals[SOURCE_ENABLED] = g_signal_new ("source-enabled", GCAL_TYPE_MANAGER, G_SIGNAL_RUN_LAST,
+                                          G_STRUCT_OFFSET (GcalManagerClass, source_enabled),
+                                          NULL, NULL, NULL,
+                                          G_TYPE_NONE, 2, E_TYPE_SOURCE, G_TYPE_BOOLEAN);
+
   signals[LOAD_COMPLETED] = g_signal_new ("load-completed", GCAL_TYPE_MANAGER, G_SIGNAL_RUN_LAST,
                                           G_STRUCT_OFFSET (GcalManagerClass, load_completed),
                                           NULL, NULL, NULL,
@@ -675,6 +686,34 @@ gcal_manager_class_init (GcalManagerClass *klass)
                                            G_STRUCT_OFFSET (GcalManagerClass, query_completed),
                                            NULL, NULL, NULL,
                                            G_TYPE_NONE, 0);
+
+  signals[GOA_CLIENT_READY] = g_signal_new ("goa-client-ready", GCAL_TYPE_MANAGER, G_SIGNAL_RUN_LAST,
+                                            G_STRUCT_OFFSET (GcalManagerClass, goa_client_ready),
+                                            NULL, NULL, NULL,
+                                            G_TYPE_NONE, 1, GOA_TYPE_CLIENT);
+}
+
+static void
+gcal_manager_client_ready_cb (GObject      *source,
+                              GAsyncResult *result,
+                              gpointer      user_data)
+{
+  GcalManagerPrivate *priv = gcal_manager_get_instance_private (GCAL_MANAGER (user_data));
+  GError *error = NULL;
+
+  priv->goa_client = goa_client_new_finish (result, &error);
+  priv->goa_client_ready = TRUE;
+
+  g_signal_emit (user_data, signals[GOA_CLIENT_READY], 0, priv->goa_client);
+
+  if (error != NULL)
+    {
+      g_warning ("%s: Error retrieving GoaClient: %s",
+                 G_STRFUNC,
+                 error->message);
+
+      g_error_free (error);
+    }
 }
 
 static void
@@ -702,6 +741,11 @@ gcal_manager_constructed (GObject *object)
   priv->clients = g_hash_table_new_full ((GHashFunc) e_source_hash, (GEqualFunc) e_source_equal,
                                          g_object_unref, (GDestroyNotify) free_unit_data);
 
+  /* load GOA client */
+  goa_client_new (NULL, // we won't really cancel it
+                  (GAsyncReadyCallback) gcal_manager_client_ready_cb,
+                  object);
+
   /* reading sources and schedule its connecting */
   priv->source_registry = e_source_registry_new_sync (NULL, &error);
   if (priv->source_registry == NULL)
@@ -795,6 +839,9 @@ gcal_manager_finalize (GObject *object)
   if (priv->shell_search_data_model != NULL)
     g_object_unref (priv->shell_search_data_model);
 
+  if (priv->goa_client != NULL)
+    g_object_unref (priv->goa_client);
+
   g_hash_table_destroy (priv->clients);
 }
 
@@ -855,6 +902,26 @@ gcal_manager_new_with_settings (GSettings *settings)
   return GCAL_MANAGER (g_object_new (GCAL_TYPE_MANAGER, "settings", settings, NULL));
 }
 
+/**
+ * gcal_manager_get_source:
+ * @manager:
+ *
+ * Retrieve a source according to it's UID. The source
+ * is referenced for thread-safety and must be unreferenced
+ * after user.
+ *
+ * Returns: (Transfer full) an {@link ESource}, or NULL.
+ **/
+ESource*
+gcal_manager_get_source (GcalManager *manager,
+                         const gchar *uid)
+{
+  GcalManagerPrivate *priv;
+  priv = gcal_manager_get_instance_private (manager);
+
+  return e_source_registry_ref_source (priv->source_registry, uid);
+}
+
 /**
  * gcal_manager_get_sources:
  * @manager:
@@ -916,6 +983,23 @@ gcal_manager_get_default_source (GcalManager *manager)
   return e_source_registry_ref_default_calendar (priv->source_registry);
 }
 
+/**
+ * gcal_manager_set_default_source:
+ * @manager: App singleton {@link GcalManager} instance
+ * @source: the new default source.
+ *
+ * Returns:
+ **/
+void
+gcal_manager_set_default_source (GcalManager *manager,
+                                 ESource     *source)
+{
+  GcalManagerPrivate *priv;
+
+  priv = gcal_manager_get_instance_private (manager);
+  e_source_registry_set_default_calendar (priv->source_registry, source);
+}
+
 icaltimezone*
 gcal_manager_get_system_timezone (GcalManager *manager)
 {
@@ -1051,6 +1135,34 @@ gcal_manager_set_query (GcalManager *manager,
                                query != NULL ? query : "#t");
 }
 
+/**
+ * gcal_manager_query_client_data:
+ *
+ * Queries for a specific data field of
+ * the {@link ECalClient}.
+ *
+ * Returns: (Transfer none) a string representing the retrieved value, or NULL.
+ */
+gchar*
+gcal_manager_query_client_data (GcalManager *manager,
+                                ESource     *source,
+                                const gchar *field)
+{
+  GcalManagerPrivate *priv;
+  GcalManagerUnit *unit;
+  gchar *out;
+
+  priv = gcal_manager_get_instance_private (manager);
+  unit = g_hash_table_lookup (priv->clients, source);
+
+  if (unit == NULL)
+    return NULL;
+
+  g_object_get (unit->client, field, &out, NULL);
+
+  return out;
+}
+
 /**
  * gcal_manager_add_source:
  * @manager: a #GcalManager
@@ -1147,6 +1259,8 @@ gcal_manager_enable_source (GcalManager *manager,
   g_strfreev (priv->disabled_sources);
   priv->disabled_sources = new_disabled_sources;
 
+  g_signal_emit (manager, signals[SOURCE_ENABLED], 0, source, TRUE);
+
   /* sync settings value */
   g_settings_set_strv (priv->settings, "disabled-sources", (const gchar * const *) priv->disabled_sources);
 }
@@ -1191,10 +1305,39 @@ gcal_manager_disable_source (GcalManager *manager,
   g_strfreev (priv->disabled_sources);
   priv->disabled_sources = new_disabled_sources;
 
+  g_signal_emit (manager, signals[SOURCE_ENABLED], 0, source, FALSE);
+
   /* sync settings value */
   g_settings_set_strv (priv->settings, "disabled-sources", (const gchar * const *) priv->disabled_sources);
 }
 
+/**
+ * gcal_manager_save_source:
+ * @manager: a #GcalManager
+ * @source: the target ESource
+ *
+ * Commit the given ESource.
+ */
+void
+gcal_manager_save_source (GcalManager *manager,
+                          ESource     *source)
+{
+  GcalManagerPrivate *priv;
+  GError *error;
+
+  priv = gcal_manager_get_instance_private (manager);
+  error = NULL;
+
+  e_source_registry_commit_source_sync (priv->source_registry, source, NULL, &error);
+
+  if (error != NULL)
+    {
+      /* FIXME: Notify the user somehow */
+      g_warning ("Error saving source: %s", error->message);
+      g_error_free (error);
+    }
+}
+
 gboolean
 gcal_manager_source_enabled (GcalManager *manager,
                              ESource     *source)
@@ -1373,7 +1516,7 @@ gcal_manager_get_event_from_shell_search (GcalManager *manager,
   GcalManagerPrivate *priv;
   time_t range_start, range_end;
   GList *l, *list = NULL;
-  GcalEventData *new_data, *data;
+  GcalEventData *data, *new_data = NULL;
   gchar *cuuid;
 
   priv = gcal_manager_get_instance_private (manager);
@@ -1403,3 +1546,27 @@ gcal_manager_get_event_from_shell_search (GcalManager *manager,
 
   return new_data;
 }
+
+gboolean
+gcal_manager_is_goa_client_ready (GcalManager *manager)
+{
+  GcalManagerPrivate *priv;
+
+  g_return_val_if_fail (GCAL_IS_MANAGER (manager), FALSE);
+
+  priv = gcal_manager_get_instance_private (manager);
+
+  return priv->goa_client_ready;
+}
+
+GoaClient*
+gcal_manager_get_goa_client (GcalManager *manager)
+{
+  GcalManagerPrivate *priv;
+
+  g_return_val_if_fail (GCAL_IS_MANAGER (manager), FALSE);
+
+  priv = gcal_manager_get_instance_private (manager);
+
+  return priv->goa_client;
+}
