@@ -29,6 +29,7 @@
 #include "gcal-event-widget.h"
 #include "gcal-edit-dialog.h"
 #include "gcal-enum-types.h"
+#include "gcal-source-dialog.h"
 
 #include <glib/gi18n.h>
 
@@ -109,7 +110,8 @@ typedef struct
 
   /* calendar management */
   GtkWidget           *calendar_popover;
-  GMenu               *calendar_menu;
+  GtkWidget           *calendar_listbox;
+  GtkWidget           *source_dialog;
   gint                 refresh_timeout;
   gint                 refresh_timeout_id;
 
@@ -141,6 +143,10 @@ static void           search_event_selected              (GcalSearchView      *s
                                                           icaltimetype        *date,
                                                           gpointer             user_data);
 
+static gint           calendar_listbox_sort_func         (GtkListBoxRow       *row1,
+                                                          GtkListBoxRow       *row2,
+                                                          gpointer             user_data);
+
 static void           load_geometry                      (GcalWindow          *window);
 
 static gboolean       save_geometry                      (gpointer             user_data);
@@ -168,6 +174,10 @@ static void           place_new_event_widget             (GcalWindow          *w
 static void           close_new_event_widget             (GtkButton           *button,
                                                           gpointer             user_data);
 
+static void           new_event_entry_text_changed       (GObject             *object,
+                                                          GParamSpec          *pspec,
+                                                          gpointer             user_data);
+
 static void           create_notification                (GcalWindow          *window,
                                                           gchar               *message,
                                                           gchar               *button_label);
@@ -181,12 +191,29 @@ static void           add_source                         (GcalManager         *m
                                                           gboolean             enabled,
                                                           gpointer             user_data);
 
+
+static GtkWidget*     make_row_from_source               (GcalWindow          *window,
+                                                          ESource             *source);
+
 static void           remove_source                      (GcalManager         *manager,
                                                           ESource             *source,
                                                           gpointer             user_data);
 
-static void           on_calendar_toggled                (GSimpleAction       *action,
-                                                          GVariant            *value,
+static void           show_source_dialog                 (GtkButton           *button,
+                                                          gpointer             user_data);
+
+static void           source_row_activated               (GtkListBox          *listbox,
+                                                          GtkListBoxRow       *row,
+                                                          gpointer             user_data);
+
+static void           source_enabled                     (GcalManager         *manager,
+                                                          ESource             *source,
+                                                          gboolean             enabled,
+                                                          gpointer             user_data);
+
+
+static void           on_calendar_toggled                (GObject             *object,
+                                                          GParamSpec          *pspec,
                                                           gpointer             user_data);
 
 static gboolean       refresh_sources                    (GcalWindow          *window);
@@ -227,7 +254,7 @@ static void           remove_event                       (GtkWidget           *n
                                                           GParamSpec          *spec,
                                                           gpointer             user_data);
 
-static void           undo_remove_event                  (GtkButton           *button,
+static void           undo_remove_action                 (GtkButton           *button,
                                                           gpointer             user_data);
 
 static void           gcal_window_constructed            (GObject             *object);
@@ -407,6 +434,22 @@ search_event_selected (GcalSearchView *search_view,
   gcal_window_set_search_mode (GCAL_WINDOW (user_data), FALSE);
 }
 
+static gint
+calendar_listbox_sort_func (GtkListBoxRow *row1,
+                            GtkListBoxRow *row2,
+                            gpointer       user_data)
+{
+  ESource *source1, *source2;
+
+  source1 = g_object_get_data (G_OBJECT (row1), "source");
+  source2 = g_object_get_data (G_OBJECT (row2), "source");
+
+  if (source1 == NULL && source2 == NULL)
+    return 0;
+
+  return g_strcmp0 (e_source_get_display_name (source1), e_source_get_display_name (source2));
+}
+
 static void
 load_geometry (GcalWindow *window)
 {
@@ -632,7 +675,10 @@ prepare_new_event_widget (GcalWindow *window)
 
   /* setting title */
   tm_date = icaltimetype_to_tm (priv->event_creation_data->start_date);
-  e_utf8_strftime_fix_am_pm (start, 64, "%B %d", &tm_date);
+  /* Translators:
+   * this is the format string for representing a date consisting of a month name
+   * and a date of month. */
+  e_utf8_strftime_fix_am_pm (start, 64, C_("event date format", "%B %d"), &tm_date);
   title_date = g_strdup_printf (_("New Event on %s"), start);
 
   gtk_label_set_text (GTK_LABEL (priv->new_event_title_label),
@@ -673,6 +719,34 @@ close_new_event_widget (GtkButton *button,
   set_new_event_mode (GCAL_WINDOW (user_data), FALSE);
 }
 
+static void
+new_event_entry_text_changed (GObject    *object,
+                              GParamSpec *pspec,
+                              gpointer    user_data)
+{
+  GcalWindowPrivate *priv;
+  static gboolean blocked = TRUE;
+  gint length;
+
+  g_return_if_fail (user_data);
+  priv = gcal_window_get_instance_private (GCAL_WINDOW (object));
+
+  length = g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY (user_data)), -1);
+
+  gtk_widget_set_sensitive (priv->new_event_create_button, length > 0);
+
+  if (length > 0 && blocked)
+    {
+      g_signal_handlers_unblock_by_func (user_data, create_event, object);
+      blocked = FALSE;
+    }
+  else if (length < 1 && !blocked)
+    {
+      g_signal_handlers_block_by_func (user_data, create_event, object);
+      blocked = TRUE;
+    }
+}
+
 /**
  * create_notification: Internal method for creating a notification
  * @window:
@@ -690,7 +764,7 @@ create_notification (GcalWindow *window,
   priv = gcal_window_get_instance_private (window);
 
   /* notification content */
-  gtk_label_set_text (GTK_LABEL (priv->notification_label), message);
+  gtk_label_set_markup (GTK_LABEL (priv->notification_label), message);
   gtk_widget_show_all (priv->notification);
 
   if (button_label != NULL)
@@ -728,38 +802,76 @@ add_source (GcalManager *manager,
             gpointer     user_data)
 {
   GcalWindowPrivate *priv;
-
-  GdkRGBA color;
-  GdkPixbuf *pix;
-  GMenuItem *item;
-  GSimpleAction *action;
-
-  gchar *item_name;
+  GtkWidget *row;
 
   priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
 
-  /* create the action itself */
-  action = g_simple_action_new_stateful (e_source_get_uid (source), NULL, g_variant_new_boolean (enabled));
-  g_signal_connect (action, "change-state", G_CALLBACK (on_calendar_toggled), user_data);
-  g_action_map_add_action (G_ACTION_MAP (user_data), G_ACTION (action));
+  row = make_row_from_source (GCAL_WINDOW (user_data), source);
+  g_object_set_data (G_OBJECT (row), "source", source);
 
-  /* retrieve the source's color & build item name */
-  item_name = g_strdup_printf ("%s", e_source_get_uid (source));
+  gtk_container_add (GTK_CONTAINER (priv->calendar_listbox), row);
+}
+
+/**
+ * make_row_from_source:
+ *
+ * Create a GtkListBoxRow for a given
+ * ESource.
+ *
+ * Returns: (transfer full) the new row
+ */
+static GtkWidget*
+make_row_from_source (GcalWindow *window,
+                      ESource    *source)
+{
+  GcalWindowPrivate *priv;
+  GtkWidget *label, *icon, *separator, *checkbox, *box, *row;
+  GtkStyleContext *context;
+  GdkPixbuf *pixbuf;
+  GdkRGBA color;
+
+  priv = gcal_window_get_instance_private (window);
+  row = gtk_list_box_row_new ();
+
+  /* apply some nice styling */
+  context = gtk_widget_get_style_context (row);
+  gtk_style_context_add_class (context, "button");
+  gtk_style_context_add_class (context, "flat");
+  gtk_style_context_add_class (context, "menuitem");
+
+  /* main box */
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_container_set_border_width (GTK_CONTAINER (box), 6);
+
+  /* source color icon */
   get_color_name_from_source (source, &color);
-  pix = gcal_get_pixbuf_from_color (&color, 16);
+  pixbuf = gcal_get_pixbuf_from_color (&color, 16);
+  icon = gtk_image_new_from_pixbuf (pixbuf);
 
-  /* create the menu item */
-  item = g_menu_item_new (e_source_get_display_name (source), item_name);
-  g_menu_item_set_attribute_value (item, "uid", g_variant_new_string (e_source_get_uid (source)));
-  g_menu_item_set_icon (item, G_ICON (pix));
-  g_menu_append_item (priv->calendar_menu, item);
+  /* source name label */
+  label = gtk_label_new (e_source_get_display_name (source));
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_widget_set_hexpand (label, TRUE);
 
-  /* HACK: show images of the popover menu */
-  fix_popover_menu_icons (GTK_POPOVER (priv->calendar_popover));
+  /* vertical separator */
+  separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
 
-  g_object_unref (pix);
-  g_object_unref (item);
-  g_free (item_name);
+  /* checkbox */
+  checkbox = gtk_check_button_new ();
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), gcal_manager_source_enabled (priv->manager, source));
+  g_signal_connect (checkbox, "notify::active", G_CALLBACK (on_calendar_toggled), window);
+
+  gtk_container_add (GTK_CONTAINER (box), icon);
+  gtk_container_add (GTK_CONTAINER (box), label);
+  gtk_container_add (GTK_CONTAINER (box), separator);
+  gtk_container_add (GTK_CONTAINER (box), checkbox);
+  gtk_container_add (GTK_CONTAINER (row), box);
+
+  gtk_widget_show_all (row);
+
+  g_object_unref (pixbuf);
+
+  return row;
 }
 
 static void
@@ -767,89 +879,116 @@ remove_source (GcalManager *manager,
                ESource     *source,
                gpointer     user_data)
 {
-  GcalWindowPrivate *priv;
-  gboolean source_found;
-  gint n_items;
-  gint i;
+  GcalWindowPrivate *priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
+  GList *children, *aux;
 
-  priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
-  n_items = g_menu_model_get_n_items (G_MENU_MODEL (priv->calendar_menu));
-  source_found = FALSE;
+  children = gtk_container_get_children (GTK_CONTAINER (priv->calendar_listbox));
 
-  for (i = 0; i < n_items; i++)
+  for (aux = children; aux != NULL; aux = aux->next)
     {
-      GMenuAttributeIter *iter;
+      ESource *child_source = g_object_get_data (G_OBJECT (aux->data), "source");
 
-      iter = g_menu_model_iterate_item_attributes (G_MENU_MODEL (priv->calendar_menu), i);
-
-      /* look for 'uid' attribute */
-      while (g_menu_attribute_iter_next (iter))
+      if (child_source != NULL && child_source == source)
         {
-          if (g_strcmp0 (g_menu_attribute_iter_get_name (iter), "uid") == 0)
-            {
-              GVariant *uid;
-              uid = g_menu_attribute_iter_get_value (iter);
-
-              /* if we find the item with uid == source::uid, remove it */
-              if (g_strcmp0 (g_variant_get_string (uid, NULL), e_source_get_uid (source)) == 0)
-                {
-                  g_menu_remove (priv->calendar_menu, i);
-                  source_found = TRUE;
-                  break;
-                }
-            }
-        }
-
-      g_object_unref (iter);
-
-      if (source_found)
-        break;
-    }
-
-  /* remove the action */
-  g_action_map_remove_action (G_ACTION_MAP (user_data), e_source_get_uid (source));
-}
-
-static void
-on_calendar_toggled (GSimpleAction *action,
-                     GVariant      *value,
-                     gpointer       user_data)
-{
-  GcalWindowPrivate *priv;
-  ESource *source;
-  GList *l;
-  GList *aux;
-
-  priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
-
-  /* lookup source */
-  source = NULL;
-  l = gcal_manager_get_sources_connected (priv->manager);
-
-  for (aux = l; aux != NULL; aux = aux->next)
-    {
-      ESource *tmp;
-      tmp = (ESource *) aux->data;
-
-      if (g_strcmp0 (e_source_get_uid (tmp), g_action_get_name (G_ACTION (action))) == 0)
-        {
-          source = tmp;
+          gtk_widget_destroy (aux->data);
           break;
         }
     }
 
-  g_list_free (l);
+  g_list_free (children);
+}
+
+static void
+show_source_dialog (GtkButton *button,
+                    gpointer   user_data)
+{
+  GcalWindowPrivate *priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
+
+  gcal_source_dialog_set_mode (GCAL_SOURCE_DIALOG (priv->source_dialog), GCAL_SOURCE_DIALOG_MODE_NORMAL);
+
+  gtk_widget_hide (priv->calendar_popover);
+
+  gtk_dialog_run (GTK_DIALOG (priv->source_dialog));
+}
+
+static void
+source_row_activated (GtkListBox    *listbox,
+                      GtkListBoxRow *row,
+                      gpointer       user_data)
+{
+  GcalWindowPrivate *priv;
+  ESource *source;
+
+  priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
+  source = g_object_get_data (G_OBJECT (row), "source");
 
   if (source == NULL)
     return;
 
-  /* toggle source visibility */
-  if (g_variant_get_boolean (value))
+  gcal_source_dialog_set_source (GCAL_SOURCE_DIALOG (priv->source_dialog), source);
+  gcal_source_dialog_set_mode (GCAL_SOURCE_DIALOG (priv->source_dialog), GCAL_SOURCE_DIALOG_MODE_EDIT);
+
+  gtk_widget_hide (priv->calendar_popover);
+
+  gtk_dialog_run (GTK_DIALOG (priv->source_dialog));
+}
+
+static void
+source_enabled (GcalManager *manager,
+                ESource     *source,
+                gboolean     enabled,
+                gpointer     user_data)
+{
+  GcalWindowPrivate *priv;
+  GList *children, *aux;
+
+  priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
+  children = gtk_container_get_children (GTK_CONTAINER (priv->calendar_listbox));
+
+  for (aux = children; aux != NULL; aux = aux->next)
+    {
+      ESource *child_source = g_object_get_data (G_OBJECT (aux->data), "source");
+
+      /*
+       * Here we destroy the row and re-add it because there is no
+       * way to actually bind the ::enabled state of source. It behaves
+       * like a construct-only property, and we need to re-create the
+       * row if there's any changes.
+       */
+      if (child_source != NULL && child_source == source)
+        {
+          gtk_widget_destroy (aux->data);
+          add_source (manager, source, enabled, user_data);
+          break;
+        }
+    }
+
+  g_list_free (children);
+}
+
+static void
+on_calendar_toggled (GObject    *object,
+                     GParamSpec *pspec,
+                     gpointer    user_data)
+{
+  GcalWindowPrivate *priv;
+  gboolean active;
+  GtkWidget *row;
+  ESource *source;
+
+  priv = gcal_window_get_instance_private (GCAL_WINDOW (user_data));
+  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (object));
+  row = gtk_widget_get_parent (gtk_widget_get_parent (GTK_WIDGET (object)));
+  source = g_object_get_data (G_OBJECT (row), "source");
+
+  if (source == NULL)
+    return;
+
+  /* Enable/disable the toggled calendar */
+  if (active)
     gcal_manager_enable_source (priv->manager, source);
   else
     gcal_manager_disable_source (priv->manager, source);
-
-  g_simple_action_set_state (action, value);
 }
 
 static gboolean
@@ -1143,8 +1282,8 @@ remove_event (GtkWidget  *notification,
 }
 
 static void
-undo_remove_event (GtkButton *button,
-                   gpointer   user_data)
+undo_remove_action (GtkButton *button,
+                    gpointer   user_data)
 {
   GcalWindowPrivate *priv;
   gchar *uuid;
@@ -1162,11 +1301,7 @@ undo_remove_event (GtkButton *button,
       g_clear_pointer (&(priv->event_to_delete), g_free);
       g_list_free (widgets);
       g_free (uuid);
-
-      g_source_remove (priv->notification_timeout);
-      hide_notification (GCAL_WINDOW (user_data), NULL);
     }
-
 }
 
 static gboolean
@@ -1253,7 +1388,9 @@ gcal_window_class_init(GcalWindowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, search_bar);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, search_button);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, calendars_button);
+  gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, calendar_listbox);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, calendar_popover);
+  gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, source_dialog);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, search_entry);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, back_button);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, today_button);
@@ -1276,6 +1413,9 @@ gcal_window_class_init(GcalWindowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, notification_action_button);
   gtk_widget_class_bind_template_child_private (widget_class, GcalWindow, notification_close_button);
 
+  gtk_widget_class_bind_template_callback (widget_class, show_source_dialog);
+  gtk_widget_class_bind_template_callback (widget_class, source_row_activated);
+
   gtk_widget_class_bind_template_callback (widget_class, key_pressed);
   gtk_widget_class_bind_template_callback (widget_class, search_toggled);
   gtk_widget_class_bind_template_callback (widget_class, search_changed);
@@ -1285,13 +1425,14 @@ gcal_window_class_init(GcalWindowClass *klass)
   /* Event removal related */
   gtk_widget_class_bind_template_callback (widget_class, hide_notification);
   gtk_widget_class_bind_template_callback (widget_class, remove_event);
-  gtk_widget_class_bind_template_callback (widget_class, undo_remove_event);
+  gtk_widget_class_bind_template_callback (widget_class, undo_remove_action);
 
   /* Event creation related */
   gtk_widget_class_bind_template_callback (widget_class, create_event);
   gtk_widget_class_bind_template_callback (widget_class, create_event_detailed_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_new_event_widget);
   gtk_widget_class_bind_template_callback (widget_class, close_new_event_widget);
+  gtk_widget_class_bind_template_callback (widget_class, new_event_entry_text_changed);
   gtk_widget_class_bind_template_callback (widget_class, event_activated);
 
   /* Syncronization related */
@@ -1306,6 +1447,13 @@ gcal_window_init (GcalWindow *self)
 {
   GcalWindowPrivate *priv = gcal_window_get_instance_private (self);
 
+  /* source dialog */
+  priv->source_dialog = gcal_source_dialog_new ();
+  gtk_window_set_transient_for (GTK_WINDOW (priv->source_dialog), GTK_WINDOW (self));
+
+  g_object_bind_property (self, "application", priv->source_dialog, "application",
+                          G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+
   priv->active_date = g_new0 (icaltimetype, 1);
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -1335,6 +1483,8 @@ gcal_window_constructed (GObject *object)
   g_free (clock_format);
   g_object_unref (helper_settings);
 
+  // Prevents nameless events' creation
+  g_signal_handlers_block_by_func (priv->new_event_what_entry, create_event, object);
 
   /* header_bar: menu */
   builder = gtk_builder_new ();
@@ -1348,10 +1498,6 @@ gcal_window_constructed (GObject *object)
 
   g_object_unref (builder);
 
-  /* calendar menu */
-  priv->calendar_menu = g_menu_new ();
-  gtk_popover_bind_model (GTK_POPOVER (priv->calendar_popover), G_MENU_MODEL (priv->calendar_menu), "win");
-
   /* edit dialog initialization */
   priv->edit_dialog = gcal_edit_dialog_new (use_24h_format);
   gtk_window_set_transient_for (GTK_WINDOW (priv->edit_dialog), GTK_WINDOW (object));
@@ -1390,6 +1536,9 @@ gcal_window_constructed (GObject *object)
   priv->refresh_timeout_id = g_timeout_add (FAST_REFRESH_TIMEOUT, (GSourceFunc) refresh_sources, object);
 
   /* calendars popover */
+  gtk_list_box_set_sort_func (GTK_LIST_BOX (priv->calendar_listbox), (GtkListBoxSortFunc) calendar_listbox_sort_func,
+                              object, NULL);
+
   if (gcal_manager_load_completed (priv->manager))
     {
       GList *sources, *l;
@@ -1444,9 +1593,11 @@ gcal_window_set_property (GObject      *object,
     case PROP_MANAGER:
       priv->manager = g_value_get_pointer (value);
       g_signal_connect (priv->manager, "source-added", G_CALLBACK (add_source), object);
+      g_signal_connect (priv->manager, "source-enabled", G_CALLBACK (source_enabled), object);
       g_signal_connect (priv->manager, "source-removed", G_CALLBACK (remove_source), object);
 
       gcal_year_view_set_manager (GCAL_YEAR_VIEW (priv->year_view), priv->manager);
+      gcal_source_dialog_set_manager (GCAL_SOURCE_DIALOG (priv->source_dialog), priv->manager);
       update_current_date (GCAL_WINDOW (object));
       return;
     }
